/*
 *  SimpleNativeRenderer.java
 *  ARToolKit5
 *
 *  Disclaimer: IMPORTANT:  This Daqri software is supplied to you by Daqri
 *  LLC ("Daqri") in consideration of your agreement to the following
 *  terms, and your use, installation, modification or redistribution of
 *  this Daqri software constitutes acceptance of these terms.  If you do
 *  not agree with these terms, please do not use, install, modify or
 *  redistribute this Daqri software.
 *
 *  In consideration of your agreement to abide by the following terms, and
 *  subject to these terms, Daqri grants you a personal, non-exclusive
 *  license, under Daqri's copyrights in this original Daqri software (the
 *  "Daqri Software"), to use, reproduce, modify and redistribute the Daqri
 *  Software, with or without modifications, in source and/or binary forms;
 *  provided that if you redistribute the Daqri Software in its entirety and
 *  without modifications, you must retain this notice and the following
 *  text and disclaimers in all such redistributions of the Daqri Software.
 *  Neither the name, trademarks, service marks or logos of Daqri LLC may
 *  be used to endorse or promote products derived from the Daqri Software
 *  without specific prior written permission from Daqri.  Except as
 *  expressly stated in this notice, no other rights or licenses, express or
 *  implied, are granted by Daqri herein, including but not limited to any
 *  patent rights that may be infringed by your derivative works or by other
 *  works in which the Daqri Software may be incorporated.
 *
 *  The Daqri Software is provided by Daqri on an "AS IS" basis.  DAQRI
 *  MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 *  THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE, REGARDING THE DAQRI SOFTWARE OR ITS USE AND
 *  OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 *
 *  IN NO EVENT SHALL DAQRI BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 *  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 *  MODIFICATION AND/OR DISTRIBUTION OF THE DAQRI SOFTWARE, HOWEVER CAUSED
 *  AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 *  STRICT LIABILITY OR OTHERWISE, EVEN IF DAQRI HAS BEEN ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  Copyright 2015 Daqri, LLC.
 *  Copyright 2011-2015 ARToolworks, Inc.
 *
 *  Author(s): Julian Looser, Philip Lamb
 *
 */

package org.artoolkit.ar.samples.ARSimpleNativeCars;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import org.artoolkit.ar.base.FPSCounter;
import org.artoolkit.ar.base.rendering.ARRenderer;

import android.util.Log;

public class SimpleNativeRenderer extends ARRenderer {
	
    // Load the native libraries.
    static {
    	System.loadLibrary("c++_shared");
		
    	System.loadLibrary("ARWrapper");	  
    	System.loadLibrary("ARWrapperNativeCarsExample");	    	
    }
    
    public static native void demoInitialise();
    public static native void demoShutdown();
    public static native void demoSurfaceCreated();
    public static native void demoSurfaceChanged(int w, int h);
    public static native void demoDrawFrame();  
    
    private FPSCounter counter = new FPSCounter();
    
	/**
     * By overriding {@link configureARScene}, the markers and other settings can be configured
     * after the native library is initialised, but prior to the rendering actually starting.
     * Note that this does not run on the OpenGL thread. Use onSurfaceCreated/demoSurfaceCreated
     * to do OpenGL initialisation.
     */
    @Override
	public boolean configureARScene() {
    	SimpleNativeRenderer.demoInitialise();
    	return true;
    }
    
    @Override
    public void onSurfaceChanged(GL10 gl, int w, int h) {       
    	super.onSurfaceChanged(gl, w, h);
    	SimpleNativeRenderer.demoSurfaceChanged(w, h);        
    }

    @Override
    public void onSurfaceCreated(GL10 gl, EGLConfig config) {
    	super.onSurfaceCreated(gl, config);
    	SimpleNativeRenderer.demoSurfaceCreated();
    }
    
    @Override
    public void draw(GL10 gl) {
    	SimpleNativeRenderer.demoDrawFrame();
    	
    	if (counter.frame()) Log.i("demo", counter.toString());
    	
    }
    
}

